#! /usr/bin/env python
#
#   Copyright (C) 2013 GC3, University of Zurich
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'

import unittest
from test import config_cluster_name, config_cloud_ec2_url,\
    config_cloud_ec2_region, config_cloud_ec2_access_key,\
    config_cloud_ec2_secret_key, config_login_user_key_private,\
    config_login_image_user, config_login_image_user_sudo,\
    config_login_image_sudo, config_setup_playbook_path, config_cloud_name,\
    config_setup_frontend_groups, config_setup_compute_groups
from elasticluster.providers.cloud_providers import BotoCloudProvider
from elasticluster.providers.setup_providers import AnsibleSetupProvider
from elasticluster.conf import Configurator
from elasticluster.cluster import Cluster, Node


from mock import Mock, MagicMock


class TestCluster(unittest.TestCase):

    def test_init(self):
        cloud_provider = BotoCloudProvider(config_cloud_ec2_url, config_cloud_ec2_region, config_cloud_ec2_access_key, config_cloud_ec2_secret_key)
        setup_provider = AnsibleSetupProvider(config_login_user_key_private, config_login_image_user, config_login_image_user_sudo, config_login_image_sudo, config_setup_playbook_path, config_setup_frontend_groups, config_setup_compute_groups)
        
        frontend_amount = 1
        compute_amount = 2
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, frontend_amount, compute_amount, Configurator())
        
        assert len(cluster.frontend_nodes) == frontend_amount
        assert len(cluster.compute_nodes) == compute_amount
        
        for node in cluster.frontend_nodes:
            assert node.type == Node.frontend_type
        
        for node in cluster.compute_nodes:
            assert node.type == Node.compute_type
        
        

    def test_add_node(self):
        cloud_provider = BotoCloudProvider(config_cloud_ec2_url, config_cloud_ec2_region, config_cloud_ec2_access_key, config_cloud_ec2_secret_key)
        setup_provider = AnsibleSetupProvider(config_login_user_key_private, config_login_image_user, config_login_image_user_sudo, config_login_image_sudo, config_setup_playbook_path, config_setup_frontend_groups, config_setup_compute_groups)
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, 1, 2, Configurator())
        
        frontend_amount = len(cluster.frontend_nodes)
        frontend_node = cluster.add_node(Node.frontend_type)
        
        assert frontend_amount == (len(cluster.frontend_nodes) - 1)
        assert frontend_node == cluster.frontend_nodes[-1]
        assert frontend_node.type == Node.frontend_type
        
        compute_amount = len(cluster.compute_nodes)
        compute_node = cluster.add_node(Node.compute_type)
        
        assert compute_amount == (len(cluster.compute_nodes) - 1)
        assert compute_node == cluster.compute_nodes[-1]
        assert compute_node.type == Node.compute_type


    def test_remove_node(self):
        cloud_provider = BotoCloudProvider(config_cloud_ec2_url, config_cloud_ec2_region, config_cloud_ec2_access_key, config_cloud_ec2_secret_key)
        setup_provider = AnsibleSetupProvider(config_login_user_key_private, config_login_image_user, config_login_image_user_sudo, config_login_image_sudo, config_setup_playbook_path, config_setup_frontend_groups, config_setup_compute_groups)
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, 1, 2, Configurator())
        
        frontend_amount = len(cluster.frontend_nodes)
        frontend_node = cluster.frontend_nodes[-1]
        frontend_node_name = frontend_node.name
        cluster.remove_node(frontend_node)
        
        assert frontend_amount == (len(cluster.frontend_nodes) + 1)
        for node in cluster.frontend_nodes:
            assert node.name != frontend_node_name

        compute_amount = len(cluster.compute_nodes)
        compute_node = cluster.compute_nodes[-1]
        compute_node_name = compute_node.name
        cluster.remove_node(compute_node)
        
        assert compute_amount == (len(cluster.compute_nodes) + 1)
        for node in cluster.compute_nodes:
            assert node.name != compute_node_name
    
    
    def test_start(self):
        cloud_provider = MagicMock()
        cloud_provider.start_instance.return_value = u'test-id'
        cloud_provider.get_ips.return_value = ('127.0.0.1', '127.0.0.1')
        running_states = [True, True, True, True, True, False, False, False, False, False]
        def side_effect_is_instance_running(id):
            return running_states.pop()
        cloud_provider.is_instance_running.side_effect = side_effect_is_instance_running
        setup_provider = MagicMock()
        setup_provider.setup_cluster.return_value = True
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, 1, 2, Configurator())
        cluster._storage = MagicMock()

        cluster.start()
        
        cluster._storage.dump_cluster.assert_called_once_with(cluster)

        for node in cluster.frontend_nodes + cluster.compute_nodes:
            assert node.instance_id == u'test-id'
            assert node.ip_public == '127.0.0.1'
            assert node.ip_private == '127.0.0.1'
            
    
    def test_stop(self):
        cloud_provider = MagicMock()
        cloud_provider.start_instance.return_value = u'test-id'
        cloud_provider.get_ips.return_value = ('127.0.0.1', '127.0.0.1')
        running_states = [True, True, True, True, True, False, False, False, False, False]
        def side_effect_is_instance_running(id):
            return running_states.pop()
        cloud_provider.is_instance_running.side_effect = side_effect_is_instance_running
        setup_provider = MagicMock()
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, 1, 2, Configurator())
        
        
        for node in cluster.frontend_nodes + cluster.compute_nodes:
            node.instance_id = u'test-id'
        
        cluster._storage = MagicMock()
            
        cluster.stop()
        

        cloud_provider.stop_instance.assert_called_with(u'test-id')
        cluster._storage.delete_cluster.assert_called_once_with(cluster.name)

    
    def test_load_from_storage(self):
        cloud_provider = MagicMock()
        cloud_provider.is_instance_running.return_value = True
        cloud_provider.get_ips.return_value = ('127.0.0.1', '127.0.0.1')
        setup_provider = MagicMock()
        storage = MagicMock()
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, 1, 2, Configurator())
        cluster._storage = storage
        cluster.load_from_storage()
        
        storage.load_cluster.assert_called_once_with(cluster)

    
    def setup(self):
        cloud_provider = MagicMock()
        setup_provider = MagicMock()
        setup_provider.setup_cluster.return_value = True
        
        cluster = Cluster(config_cluster_name, config_cloud_name, cloud_provider, setup_provider, 1, 2, Configurator())
        
        cluster.setup()
        
        setup_provider.setup_cluster.assert_called_once_with(cluster)
        
        
        
        
        
        
        
        
        
        


#! /usr/bin/env python
#
# Copyright (C) 2013 GC3, University of Zurich
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'

import sys

from elasticluster.conf import Configurator
from elasticluster.conf import Configuration
from elasticluster import log
from elasticluster.exceptions import ClusterNotFound, ConfigurationError,\
    ImageError, SecurityGroupError


class AbstractCommand():
    """
    Defines the general contract every command has to fullfill in
    order to be recognized by the arguments list and executed
    afterwards.
    """

    def __init__(self, params):
        """
        A reference to the parameters of the command line will be
        passed here to adjust the functionality of the command
        properly.
        """
        self.params = params

    def setup(self, subparsers):
        """
        This method handles the setup of the subcommand. In order to
        do so, every command has to add a parser to the subparsers
        reference given as parameter. The following example is the
        minimum implementation of such a setup procedure: parser =
        subparsers.add_parser("start")
        parser.set_defaults(func=self.execute)
        """
        pass

    def execute(self):
        """
        This method is executed after a command was recognized and may
        vary in its behavior.
        """
        pass

    def __call__(self):
        return self.execute()

    def pre_run(self):
        """
        Overrides this method to execute any pre-run code, especially
        to check any command line options.
        """
        pass


def cluster_summary(cluster):
    if cluster.frontend_nodes:
        frontend = cluster.frontend_nodes[0]
        return """
Cluster name:     %s
Cluster template: %s
Frontend nodes: %3d
Compute nodes:  %3d

To login on the frontend node, run the command:

    ssh %s@%s -i %s
""" % (cluster.name, cluster.template, len(cluster.compute_nodes),
       len(cluster.frontend_nodes), frontend.image_user, frontend.ip_public,
       frontend.user_key_private)
    else:
        # Invalid/not complete cluster!
        return """
INCOMPLETE CLUSTER! MISSING FRONTEND NODE!
Cluster name:     %s
Cluster template: %s
Compute nodes:    %d""" % (cluster.name, cluster.template, len(cluster.compute_nodes))

class Start(AbstractCommand):
    """
    Handles the start of a cluster. The cluster parameters are mostly
    read from a configuration file in order to keep the parameter set
    to a minimum.
    """

    def setup(self, subparsers):
        """
        @see abstract_command contract
        """
        parser = subparsers.add_parser("start")
        parser.set_defaults(func=self)
        parser.add_argument('cluster',
                            help="Type of cluster. It refers to a "
                            "configuration stanza [cluster/<name>]")
        parser.add_argument('-v', '--verbose', action='count', default=0,
                            help="Increase verbosity.")
        parser.add_argument('-n', '--name', dest='cluster_name',
                            help='Name of the cluster.')
        parser.add_argument('-c', '--compute-nodes',
                            help='Number of compute nodes.')
        parser.add_argument('--no-setup', action="store_true", default=False,
                            help="Only start the cluster, do not configure it")
                            

    def pre_run(self):
        if self.params.compute_nodes:
            try:
                self.params.compute_nodes = int(self.params.compute_nodes)
            except ValueError:
                pass

    def execute(self):
        """
        Starts a new cluster.
        """
        
        cluster_template = self.params.cluster
        if self.params.cluster_name:
            cluster_name = self.params.cluster_name
        else:
            cluster_name = self.params.cluster

        # First, check if the cluster is already created.
        try:
            cluster = Configurator().load_cluster(cluster_name)
        except ClusterNotFound, ex:
            extra_conf = dict()
            if self.params.compute_nodes:
                extra_conf['compute'] = self.params.compute_nodes

            if self.params.cluster_name:
                extra_conf['name'] = self.params.cluster_name

            try:
                cluster = Configurator().create_cluster(
                    cluster_template, **extra_conf)
            except ConfigurationError, ex:
                log.error("Cluster `%s` not found in configuration file." %
                          cluster_template)
                return

        try:
            print("Starting cluster `%s` with %d compute nodes." % (
                cluster.name, len(cluster.compute_nodes)))
            print("(this may take a while...)")
            cluster.start()
            if self.params.no_setup:
                print("NOT configuring the cluster as requested.")
            else:
                print("Configuring the cluster.")
                print("(this too may take a while...)")
                cluster.setup()
                print("Your cluster is ready!")
            print(cluster_summary(cluster))
        except (KeyError, ImageError, SecurityGroupError) as e:
            print("Your cluster could not start `%s`" % e)


class Stop(AbstractCommand):
    """
    Handles the stop of a cluster.
    """
    def setup(self, subparsers):
        """
        @see abstract_command contract
        """
        parser = subparsers.add_parser("stop")
        parser.set_defaults(func=self)
        parser.add_argument('cluster', help='name of the cluster')
        parser.add_argument('-v', '--verbose', action='count', default=0,
                            help="Increase verbosity.")
        parser.add_argument('--force', action="store_true", default=False,
                            help="Remove the cluster even if not all the nodes"
                            " have been terminated properly.")

    def execute(self):
        """
        Stops the cluster if it's running.
        """
        cluster_name = self.params.cluster
        try:
            cluster = Configurator().load_cluster(cluster_name)
        except (ClusterNotFound, ConfigurationError), ex:
            log.error("Stopping cluster %s: %s\n" %
                      (cluster_name, ex))
            return

        print("Destroying cluster `%s`" % cluster_name)
        cluster.stop(force=self.params.force)


class ResizeCluster(AbstractCommand):
    """
    Resize the cluster by adding or removing
    """
    def setup(self, subparsers):
        parser = subparsers.add_parser("resize")
        parser.set_defaults(func=self)
        parser.add_argument('cluster', help='name of the cluster')
        parser.add_argument('N', help="Number of compute nodes, or, using +n"
                            "or -n, resize by adding or removing `n` nodes.")
        parser.add_argument('-v', '--verbose', action='count', default=0,
                            help="Increase verbosity.")
        parser.add_argument('--no-setup', action="store_true", default=False,
                            help="Only start the cluster, do not configure it")

    def pre_run(self):
        self.params.nodes_to_add = 0
        self.params.nodes_to_remove = 0
        try:
            int(self.params.N)
        except ValueError:
            raise RuntimeError("Value of `N` must be an integer")

    def execute(self):
        # Get current cluster configuration
        cluster_name = self.params.cluster
        try:
            cluster = Configurator().load_cluster(cluster_name)
            cluster.update()
        except (ClusterNotFound, ConfigurationError), ex:
            log.error("Listing nodes from cluster %s: %s\n" %
                      (cluster_name, ex))
            return

        if self.params.N.startswith('+'):
            self.params.nodes_to_add = int(self.params.N)
        elif self.params.N.startswith('-'):
            self.params.nodes_to_remove = abs(int(self.params.N))
        else:
            N = int(self.params.N)
            nnodes = len(cluster.compute_nodes)
            if nnodes > N:
                self.params.nodes_to_remove = nnodes - N
            else:
                self.params.nodes_to_add = N - nnodes

        if self.params.nodes_to_add:
            print("Adding %d nodes to the cluster"
                  "" % self.params.nodes_to_add)
        for i in range(self.params.nodes_to_add):
            cluster.add_node('compute')

        if self.params.nodes_to_remove:
            print("Removing %d nodes from the cluster."
                  "" % self.params.nodes_to_remove)
        for i in range(self.params.nodes_to_remove):
            node = cluster.compute_nodes.pop()
            if node in cluster.compute_nodes:
                cluster.remove_node(node)
            node.stop()

        cluster.start()
        if self.params.no_setup:
            print("NOT configuring the cluster as requested.")
        else:
            print("Reconfiguring the cluster.")
            cluster.setup()
        print(cluster_summary(cluster))


class ListClusters(AbstractCommand):
    """
    Handles the listing of all clusters.
    """
    def setup(self, subparsers):
        parser = subparsers.add_parser("list")
        parser.set_defaults(func=self)
        parser.add_argument('-v', '--verbose', action='count', default=0,
                            help="Increase verbosity.")

    def execute(self):
        storage = Configurator().create_cluster_storage()
        cluster_names = storage.get_stored_clusters()

        if not cluster_names:
            print("No clusters found.")
        else:
            print("""
The following clusters appear in your storage.
Yet, there's no guarantee that they are up and running:
""")
            for name in cluster_names:
                print("- %s " % name)


class ListNodes(AbstractCommand):
    """
    Handles the listing of information about a cluster.
    """

    def setup(self, subparsers):
        parser = subparsers.add_parser("list-nodes")
        parser.set_defaults(func=self)
        parser.add_argument('cluster', help='name of the cluster')
        parser.add_argument('-v', '--verbose', action='count', default=0,
                            help="Increase verbosity.")
        parser.add_argument(
            '-u', '--update', action='store_true', default=False,
            help="By default `elasticluster list-nodes` will not contact the "
            "EC2 provider to get up-to-date information, unless `-u` option "
            "is given.")

    def execute(self):
        """
        Lists all nodes within the specified cluster with certain
        information like id and ip.
        """
        Configuration.Instance().cluster_name = self.params.cluster
        cluster_name = self.params.cluster
        try:
            cluster = Configurator().load_cluster(cluster_name)
            if self.params.update:
                cluster.update()
        except (ClusterNotFound, ConfigurationError), ex:
            log.error("Listing nodes from cluster %s: %s\n" %
                      (cluster_name, ex))
            return

        print(cluster_summary(cluster))
        if cluster.frontend_nodes:
            print("")
            print("Frontend nodes:")
            for node in cluster.frontend_nodes:
                print("  " + str(node))

        if cluster.compute_nodes:
            print("")
            print("Compute nodes:")
            for node in cluster.compute_nodes:
                print("  " + str(node))


class SetupCluster(AbstractCommand):
    def setup(self, subparsers):
        parser = subparsers.add_parser("setup")
        parser.set_defaults(func=self)
        parser.add_argument('cluster', help='name of the cluster')
        parser.add_argument('-v', '--verbose', action='count', default=0,
                            help="Increase verbosity.")

    def execute(self):
        Configuration.Instance().cluster_name = self.params.cluster
        cluster_name = self.params.cluster
        try:
            cluster = Configurator().load_cluster(cluster_name)
            cluster.update()
        except (ClusterNotFound, ConfigurationError), ex:
            log.error("Setting up cluster %s: %s\n" %
                      (cluster_name, ex))
            return

        print("Configuring cluster `%s`..." % cluster_name)
        cluster.setup()
        print("Your cluster is ready!")
        print(cluster_summary(cluster))



#! /usr/bin/env python
#
#   Copyright (C) 2013 GC3, University of Zurich
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'

import unittest
from StringIO import StringIO
from mock import Mock
from mock import patch
from mock import MagicMock
from mock import mock_open
import ConfigParser

from elasticluster.conf import Configuration

"""
This section mocks the configuration file and defines the variables to mock it with.
We need this actually to test the classes related to configuration.
"""

config_storage_path = "~/.elasticluster/storage"

config_cloud_name = "test"
config_cloud_provider = "ec2_boto"
config_cloud_ec2_url = "http://130.60.24.4:8773/services/Cloud"
config_cloud_ec2_access_key = "test"
config_cloud_ec2_secret_key = "test"
config_cloud_ec2_region = "nova"

config_setup_name = "test"
config_setup_provider = "ansible"
config_setup_playbook_path = "~/.elasticluster/playbook"
config_setup_frontend_groups = "slurm_master"
config_setup_compute_groups = "slurm_clients"

config_login_name = "test"
config_login_image_user = "test"
config_login_image_user_sudo = "test"
config_login_image_sudo = "True"
config_login_user_key_name = "test"
config_login_user_key_private = "~/test.prv"
config_login_user_key_public = "~/test.pub"

config_cluster_name = "test"
config_cluster_cloud = config_cloud_name
config_cluster_setup_provider = config_setup_name
config_cluster_frontend = "1"
config_cluster_compute = "2"
config_cluster_login = "test"

config_frontend_image = "ami-00000"
config_frontend_security_group = "all"
config_frontend_flavor = "m1.tiny"
config_frontend_image_userdata = "none"

config_compute_image = "Ubuntu"
config_compute_security_group = "all"
config_compute_flavor = "m1.tiny"
config_compute_image_userdata = "none"

config_section_cloud = "cloud/" + config_cloud_name
config_section_cluster = "cluster/" + config_cluster_name
config_section_frontend = "cluster/" + config_cluster_name + "/frontend"
config_section_compute = "cluster/"+ config_cluster_name +"/compute"
config_section_login = "login/" + config_login_name
config_section_setup = "setup/" + config_setup_name


def setup():
    config = Mock()
    def side_effect_has_section(name):
        sections = [config_section_cloud, config_section_cluster, config_section_compute, config_section_frontend, config_section_login, config_section_setup]
        if name in sections:
            return True
        else:
            return False
    
    def side_effect_items(name):
        content = {
                   config_section_cloud :
                   [("provider", config_cloud_provider), ("ec2_url", config_cloud_ec2_url),("ec2_access_key", config_cloud_ec2_access_key), ("ec2_secret_key", config_cloud_ec2_secret_key), ("ec2_region", config_cloud_ec2_region)],
                   
                   config_section_setup:
                   [("provider", config_setup_provider), ("playbook_path", config_setup_playbook_path), ("frontend_groups", config_setup_frontend_groups), ("compute_groups", config_setup_compute_groups)],
                   
                   config_section_login:
                   [("image_user", config_login_image_user), ("image_user_sudo", config_login_image_user_sudo), ("image_sudo", config_login_image_sudo), ("user_key_name", config_login_user_key_name), ("user_key_private", config_login_user_key_private), ("user_key_public", config_login_user_key_public)],
                   
                   config_section_cluster:
                   [("cloud", config_cluster_cloud), ("setup_provider", config_cluster_setup_provider), ("frontend", config_cluster_frontend), ("compute", config_cluster_compute), ("login", config_cluster_login)],
                   
                   config_section_frontend:
                   [("image_id", config_frontend_image), ("security_group", config_frontend_security_group), ("flavor", config_frontend_flavor), ("image_userdata", config_frontend_image_userdata)],
                   
                   config_section_compute:
                   [("image_id", config_compute_image), ("security_group", config_compute_security_group), ("flavor", config_compute_flavor), ("image_userdata", config_compute_image_userdata)]
                   }
        if name in content:
            return content[name]
        else:
            raise ConfigParser.NoSectionError()
    
    config.has_section.side_effect = side_effect_has_section
    config.items.side_effect = side_effect_items
    config.read.return_vale = True
    
    Configuration.Instance()._config = config
    Configuration.Instance().cluster_name = config_cluster_name
    Configuration.Instance().storage_path = config_storage_path
    
    



#! /usr/bin/env python
#
# Copyright (C) 2013 GC3, University of Zurich
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'

import ConfigParser
import os
import sys

from elasticluster import log
from elasticluster.providers.cloud_providers import BotoCloudProvider
from elasticluster.providers.setup_providers import AnsibleSetupProvider
from elasticluster.helpers import Singleton
from elasticluster.cluster import Node, ClusterStorage
from elasticluster.exceptions import ConfigurationError, ClusterNotFound
from elasticluster.cluster import Cluster


class Configurator(object):
    """
    Responsible to create instances, which need information from the
    configuration file.
    """

    cloud_providers_map = {"ec2_boto": BotoCloudProvider, }

    setup_providers_map = {"ansible": AnsibleSetupProvider, }

    def create_cloud_provider(self, cloud_name):
        """
        Creates a new cloud provider with the needed information from
        the configuration.
        """
        config = Configuration.Instance().read_cloud_section(cloud_name)
        provider = Configurator.cloud_providers_map[config["provider"]]

        return provider(config["ec2_url"],
                        config["ec2_region"],
                        config["ec2_access_key"],
                        config["ec2_secret_key"])

    def create_cluster(self, cluster_template, **extra_args):
        """
        Creates a cluster with the needed information from the
        configuration.
        """
        try:
            config = Configuration.Instance().read_cluster_section(
                cluster_template)
        except ConfigParser.NoSectionError:
            raise ConfigurationError(
                "Cluster `%s` not found in configuration file."
                "" % cluster_template)
        config['name'] = cluster_template

        # Update with extra conf
        config.update(extra_args)
        return Cluster(cluster_template,
                       config['name'],
                       config['cloud'],
                       self.create_cloud_provider(config['cloud']),
                       self.create_setup_provider(
                           config["setup_provider"], cluster_template),
                       int(config['frontend']),
                       int(config['compute']),
                       self)  # ANTONIO: why self? Why at the end? It
                              # does not looks right

    def load_cluster(self, cluster_name):
        storage = self.create_cluster_storage()
        information = storage.load_cluster(cluster_name)

        cluster = Configurator().create_cluster(
            information['template'], name=information['name'])

        # Clear frontend and compute nodes
        cluster.compute_nodes = []
        cluster.frontend_nodes = []

        for compute in information['compute']:
            node = cluster.add_node(Node.compute_type, name=compute['name'])
            node.instance_id = compute.get('instance_id')
            node.ip_public = compute.get('ip_public')
            node.ip_private = compute.get('ip_private')

        for frontend in information['frontend']:
            node = cluster.add_node(Node.frontend_type, name=frontend['name'])
            node.instance_id = frontend.get('instance_id')
            node.ip_public = frontend.get('ip_public')
            node.ip_private = frontend.get('ip_private')

        return cluster

    def create_node(self, cluster_name, node_type, cloud_provider, name):
        """
        Creates a node with the needed information from the
        configuration file. The information of the node is specific to
        its type (e.g. a frontend node could differ from a compute
        node).
        """
        config = Configuration.Instance().read_node_section(
            cluster_name, node_type)

        return Node(name, node_type, cloud_provider, config['user_key_public'],
                    config["user_key_private"], config['user_key_name'],
                    config['image_user'], config['security_group'],
                    config['image_id'], config['flavor'],
                    config.get('image_userdata', ''))

    def create_cluster_storage(self):
        """
        Creates the storage to manage clusters.
        """
        return ClusterStorage(Configuration.Instance().storage_path)

    def create_setup_provider(self, setup_provider_name, cluster_name):
        config = Configuration.Instance().read_setup_section(
            setup_provider_name, cluster_name)
        provider = Configurator.setup_providers_map[config['provider']]

        return provider(config['user_key_private'], config['image_user'],
                        config['image_user_sudo'], config['image_sudo'],
                        config['playbook_path'], config['frontend_groups'],
                        config['compute_groups'])


class QuotelessConfigParser(ConfigParser.ConfigParser):
    """
    This implementation removes all the quotes from the value of a property.
    """
    def get(self, section, option):
        val = ConfigParser.ConfigParser.get(self, section, option)
        return val.strip('"').strip("'")

    def items(self, section):
        items = ConfigParser.ConfigParser.items(self, section)
        items_stripped = []
        for i in items:
            l = list(i)
            if l[1]:
                l[1] = l[1].strip("'").strip('"')
            items_stripped.append(tuple(l))

        return items_stripped


@Singleton
class Configuration(object):
    """
    Singleton

    The configuration class handles the global configuration file.  It
    parses the file and provides the important sections as *datatype
    undecied*
    """

    mandatory_cloud_options = ("provider", "ec2_url", "ec2_access_key",
                               "ec2_secret_key", "ec2_region")
    mandatory_cluster_options = ("cloud", "frontend", "compute",
                                 "setup_provider", "login")
    mandatory_node_options = ("image_id", "security_group", "image_userdata",
                              "flavor")
    mandatory_setup_options = ("provider", "playbook_path", "frontend_groups",
                               "compute_groups")
    mandatory_login_options = ("image_user", "image_user_sudo", "image_sudo",
                               "user_key_name", "user_key_private",
                               "user_key_public")

    config_defaults = {
        'ansible_pb_dir' : os.path.join(sys.prefix, 'share/elasticluster/providers/ansible-playbooks'),
        }

    def __init__(self):
        # will be initialized upon user input from outside
        self.file_path = None
        self.cluster_name = None
        self.storage_path = None

        self._config = QuotelessConfigParser(self.config_defaults)

    def _read_section(self, name):
        """
        Reads a section from the configuration file and returns a
        dictionary with its content
        """
        self._config.read(self.file_path)
        if self._config.has_section(name):
            return dict(self._config.items(name))
        else:
            raise ConfigParser.NoSectionError("section %s not found in "
                                              "configuration file" % name)

    def read_cluster_section(self, name):
        """
        Reads the cluster section for a given cluster name from the
        configuration file and returns its properties in a dictionary.
        """
        config = self._read_section("cluster/"+name)
        self._check_mandatory_options(
            Configuration.Instance().mandatory_cluster_options, config)

        config_login = self.read_login_section(config["login"])

        return dict(config.items() + config_login.items())

    def read_node_section(self, cluster_name, node_type):
        """
        Reads the cluster configuration from the current config file
        with the given parameters for cluster and node type.  In this
        case, the sectoins of the cluster and the specific node types
        will be merged (node_type is more specific) in order to allow
        easier configuration options.
        """

        if node_type == Node.frontend_type:
            node_type = "frontend"
        else:
            node_type = "compute"

        config_name_general = "cluster/" + cluster_name
        config_name_specific = "cluster/" + cluster_name + "/" + node_type

        # merge configuration parts from the cluster and
        # compute/frontend section
        if self._config.has_section(config_name_general):
            if self._config.has_section(config_name_specific):
                config = dict(self.read_cluster_section(cluster_name).items() +
                              self._read_section(config_name_specific).items())
            else:
                config = self.read_cluster_section(cluster_name)

            self._check_mandatory_options(
                Configuration.Instance().mandatory_node_options, config)

            return config

        else:
            raise ConfigParser.NoSectionError(
                "no configuration secton for cluster `%s`"
                " found" % cluster_name)

    def read_cloud_section(self, name):
        """
        Reads the cloud section for a given cloud name from the
        configuraiton file and returns its properties in a dictionary.
        """
        config = self._read_section("cloud/"+name)

        self._check_mandatory_options(
            Configuration.Instance().mandatory_cloud_options, config)

        return config

    def read_setup_section(self, name, cluster_name):
        """
        Reads the setup section for a given setup name from the
        configuration file and returns its properties in a dictionary
        """
        config = self._read_section("setup/"+name)
        self._check_mandatory_options(
            Configuration.Instance().mandatory_setup_options, config)

        config["playbook_path"] = os.path.expanduser(
            os.path.expanduser(config["playbook_path"]))

        login_name = self.read_cluster_section(cluster_name)["login"]

        config_login = self.read_login_section(login_name)

        return dict(config.items() + config_login.items())

    def read_login_section(self, name):
        """
        Reads the login section for the given name from the
        configuration file and returns its properties in a dictionary
        """
        config = self._read_section("login/" + name)
        self._check_mandatory_options(
            Configuration.Instance().mandatory_login_options, config)
        config["user_key_private"] = os.path.expanduser(
            os.path.expandvars(config["user_key_private"]))
        config["user_key_public"] = os.path.expanduser(
            os.path.expandvars(config["user_key_public"]))

        if (not os.path.exists(config["user_key_private"]) or
                not os.path.exists(config["user_key_public"])):
            log.warning("The key files don't exist. Please check your "
                        "configuration file `user_key_public`, "
                        "`user_key_private`.")

        return config

    def _check_mandatory_options(self, options, config):
        for o in options:
            if o not in config:
                raise ConfigurationError(
                    "could not find mandatory cloud option `%s` in "
                    "configuration file" % o)



#! /usr/bin/env python
#
# Copyright (C) 2013 GC3, University of Zurich
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'


class AbstractCloudProvider:
    """
    Defines the contract for a cloud provider to proper function with
    elasticluster.
    """

    def start_instance(self, key_name, key_path, security_group,
                       flavor, image_name, image_userdata):
        """
        Starts a new instance with the given properties and returns
        the instance id.
        """
        pass

    def stop_instance(self, instance_id):
        """
        Stops the instance with the given id gracefully.
        """
        pass

    def is_instance_running(self, instance_id):
        """
        Checks if the instance with the given id is up and running.
        """
        pass


class AbstractSetupProvider:
    """
    TODO: define...
    """

    def __init__(self):
        pass

    def setup_cluster(self, cluster):
        pass



#! /usr/bin/env python
#
# Copyright (C) 2013 GC3, University of Zurich
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'

import json
import os
import signal
import socket
import time

import paramiko

from elasticluster import log
from elasticluster.exceptions import TimeoutError, ClusterNotFound


class Cluster(object):
    """
    Handles all cluster related functionality such as start, setup,
    load, stop, storage etc.
    """
    startup_timeout = 60*10

    def __init__(self, template, name, cloud, cloud_provider, setup_provider,
                 frontend, compute, configurator, **extra):
        self.template = template
        self.name = name
        self._cloud = cloud
        self._frontend = frontend
        self._compute = compute
        self._cloud_provider = cloud_provider
        self._setup_provider = setup_provider

        self._configurator = configurator
        self._storage = configurator.create_cluster_storage()

        self.compute_nodes = []
        self.frontend_nodes = []

        # initialize nodes
        for _ in range(self._frontend):
            self.add_node(Node.frontend_type)

        for _ in range(self._compute):
            self.add_node(Node.compute_type)

    def add_node(self, node_type, name=""):
        """
        Adds a new node, but doesn't start the instance on the cloud.
        Returns the created node instance
        """
        name = ""
        if not name:
            if node_type == Node.frontend_type:
                name = "frontend" + str(len(self.frontend_nodes) + 1).zfill(3)
            elif node_type == Node.compute_type:
                name = "compute" + str(len(self.compute_nodes) + 1).zfill(3)
            else:
                log.warning("Invalid node type %s given. "
                            "Unable to add node" % node_type)
                return

        node = self._configurator.create_node(self.template, node_type,
                                              self._cloud_provider, name)
        if node_type == Node.frontend_type:
            self.frontend_nodes.append(node)
        else:
            self.compute_nodes.append(node)

        return node

    def remove_node(self, node):
        """
        Removes a node from the cluster, but does not stop it.
        """
        if node.type == Node.compute_type:
            self.compute_nodes.remove(node)
        elif node.type == Node.frontend_type:
            self.frontend_nodes.remove(node)

    def start(self):
        """
        Starts the cluster with the properties given in the
        constructor. It will create the nodes through the configurator
        and delegate all the work to them. After the identifiers of
        all instances are available, it will save the cluster throgh
        the cluster storage.
        """

        # start every node
        try:
            for node in self.frontend_nodes + self.compute_nodes:
                if node.is_alive():
                    log.warning("Not starting node %s which is "
                                "already up&running.", node.name)
                else:
                    node.start()
        except:
            log.error("Error occured during node start, stopping all nodes")
            self.stop()
            raise
            
        # dump the cluster here, so we don't loose any knowledge about nodes
        self._storage.dump_cluster(self)

        # check if all nodes are running, stop all nodes if the
        # timeout is reached
        def timeout_handler(signum, frame):
            raise TimeoutError("problems occured while starting the nodes, "
                               "timeout `%i`", Cluster.startup_timeout)

        signal.signal(signal.SIGALRM, timeout_handler)
        signal.alarm(Cluster.startup_timeout)

        try:
            starting_nodes = self.compute_nodes + self.frontend_nodes
            while starting_nodes:
                starting_nodes = [n for n in starting_nodes
                                  if not n.is_alive()]
                if starting_nodes:
                    time.sleep(5)
        except TimeoutError as timeout:
            log.error(timeout.message)
            log.error("timeout error occured: "
                      "stopping all nodes")
            self.stop()

        signal.alarm(0)

        # If we reached this point, we should have IP addresses for
        # the nodes, so update the storage file again.
        self._storage.dump_cluster(self)

        # Try to connect to each node. Run the setup action only when
        # we successfully connect to all of them.
        signal.signal(signal.SIGALRM, timeout_handler)
        signal.alarm(Cluster.startup_timeout)
        pending_nodes = self.compute_nodes + self.frontend_nodes

        try:
            while pending_nodes:
                for node in pending_nodes[:]:
                    if node.connect():
                        pending_nodes.remove(node)
                time.sleep(5)

        except TimeoutError:
            log.error("Timeout occured after trying to connect to the nodes \
                        via ssh. The nodes are running, but no connection\
                        could be established and the setup did not run. \
                        Please re-run `elasticluster setup %s`", self.name)

        signal.alarm(0)

    def stop(self, force=False):
        """
        Terminates all instances corresponding to this cluster and
        deletes the cluster storage.
        """
        for node in self.frontend_nodes + self.compute_nodes:
            try:
                node.stop()
                if node in self.compute_nodes:
                    self.compute_nodes.remove(node)
                elif node in self.frontend_nodes:
                    self.frontend_nodes.remove(node)
                else:
                    log.warning(
                        "node %s (instance id %s) is nor a compute node nor a "
                        "frontend node! Something strange happened!", 
                        node.name, node.instance_id)
            except:
                # Boto does not always raises an `Exception` class!
                log.error("could not stop instance `%s`, it might "
                          "already be down.", node.instance_id)
        if not self.frontend_nodes and not self.compute_nodes:
            log.debug("Removing cluster %s.", self.name)
            self._storage.delete_cluster(self.name)
        elif not force:
            log.warning("Not all instances have been terminated. "
                        "Please rerun the `elasticluster stop %s`", self.name)
            self._storage.dump_cluster(self)
        else:
            log.warning("Not all instances have been terminated. However, "
                        "as requested, the cluster has been force-removed.")
            self._storage.delete_cluster(self.name)

    def setup(self):
        try:
            # setup the cluster using the setup provider
            ret = self._setup_provider.setup_cluster(self)
        except Exception, e:
            log.error(
                "the setup provider was not able to setup the cluster, "
                "but the cluster is running by now. Setup provider error "
                "message: `%s`", str(e))
            ret = False

        if not ret:
            log.warning(
                "Cluster `%s` not yet configured. Please, re-run "
                "`elasticluster setup %s` and/or check your configuration",
                self.name, self.name)

        return ret

    def update(self):
        for node in self.compute_nodes + self.frontend_nodes:
            node.update_ips()
        self._storage.dump_cluster(self)


class Node(object):
    """
    Handles all the node related funcitonality such as start, stop,
    configure, etc.
    """
    frontend_type = 'frontend'
    compute_type = 'compute'

    def __init__(self, name, node_type, cloud_provider, user_key_public,
                 user_key_private, user_key_name, image_user, security_group,
                 image, flavor, image_userdata=None):
        self.name = name
        self.type = node_type
        self._cloud_provider = cloud_provider
        self.user_key_public = user_key_public
        self.user_key_private = user_key_private
        self.user_key_name = user_key_name
        self.image_user = image_user
        self.security_group = security_group
        self.image = image
        self.image_userdata = image_userdata
        self.flavor = flavor

        self.instance_id = None
        self.ip_public = None
        self.ip_private = None

    def start(self):
        """
        Starts an instance for this node on the cloud through the
        clode provider. This method is non-blocking, as soon as the
        node id is returned from the cloud provider, it will return.
        """
        log.info("Starting node %s.", self.name)
        self.instance_id = self._cloud_provider.start_instance(
            self.user_key_name, self.user_key_public, self.security_group,
            self.flavor, self.image, self.image_userdata)
        log.info("starting node with id `%s`" % self.instance_id)

    def stop(self):
        log.info("shutting down instance `%s`",
                 self.instance_id)
        self._cloud_provider.stop_instance(self.instance_id)

    def is_alive(self):
        """
        Checks if the current node is up and running in the cloud
        """
        running = False
        if not self.instance_id:
            return running

        try:
            log.debug("Getting information for instance %s",
                      self.instance_id)
            running = self._cloud_provider.is_instance_running(
                self.instance_id)
        except Exception, ex:
            log.debug("Ignoring error while looking for vm id %s: %s",
                      self.instance_id, str(ex))
        if running:
            log.info("node `%s` (instance id %s) is up and running",
                     self.name, self.instance_id)
            self.update_ips()
        else:
            log.debug("node `%s` (instance id `%s`) still building...",
                      self.name, self.instance_id)

        return running

    def connect(self):
        """
        Connect to the node via ssh and returns a paramiko.SSHClient
        object, or None if we are unable to connect.
        """
        ssh = paramiko.SSHClient()
        ssh.load_system_host_keys()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            log.debug("Trying to connect to host %s (%s)",
                      self.name, self.ip_public)
            ssh.connect(self.ip_public,
                        username=self.image_user,
                        allow_agent=True,
                        key_filename=self.user_key_private)
            log.debug("Connection to %s succeded!", self.ip_public)
            return ssh
        except socket.error, ex:
            log.debug("Host %s (%s) not reachable: %s.",
                      self.name, self.ip_public, ex)
        except paramiko.SSHException, ex:
            log.debug("Ignoring error %s connecting to %s",
                      str(ex), self.name)

        return None

    def update_ips(self):
        """
        Updates the ips of the node through the cloud provider.
        """
        if not self.ip_private or not self.ip_public:
            private, public = self._cloud_provider.get_ips(self.instance_id)
            self.ip_public = public
            self.ip_private = private

    def __str__(self):
        return "name=`%s`, id=`%s`, public_ip=`%s`, private_ip=`%s`" % (
            self.name, self.instance_id, self.ip_public, self.ip_private)


class ClusterStorage(object):
    """
    Handles the storage to save information about all the clusters
    managed by this tool.
    """

    def __init__(self, storage_dir):
        self._storage_dir = storage_dir

    def dump_cluster(self, cluster):
        """
        Saves the information of the cluster to disk in json format to
        load it later on.
        """
        db = {"name": cluster.name, "template": cluster.template}
        db["frontend"] = [
            {'instance_id': node.instance_id,
             'name': node.name,
             'ip_public': node.ip_public,
             'ip_private': node.ip_private} for node in cluster.frontend_nodes]
        db["compute"] = [
            {'instance_id': node.instance_id,
             'name': node.name,
             'ip_public': node.ip_public,
             'ip_private': node.ip_private} for node in cluster.compute_nodes]

        db_json = json.dumps(db)

        db_path = self._get_json_path(cluster.name)
        self._clear_storage(db_path)

        f = open(db_path, 'w')
        f.write(unicode(db_json))
        f.close()

    def load_cluster(self, cluster_name):
        """
        Read the storage file, create a cluster and return a `Cluster`
        object.
        """
        db_path = self._get_json_path(cluster_name)

        if not os.path.exists(db_path):
            raise ClusterNotFound("Storage file %s not found" % db_path)
        f = open(db_path, 'r')
        db_json = f.readline()

        information = json.loads(db_json)

        return information

    def delete_cluster(self, cluster_name):
        """
        Deletes the storage of a cluster.
        """
        db_file = self._get_json_path(cluster_name)
        self._clear_storage(db_file)

    def get_stored_clusters(self):
        """
        Returns a list of all stored clusters.
        """
        allfiles = os.listdir(self._storage_dir)
        db_files = []
        for fname in allfiles:
            fpath = os.path.join(self._storage_dir, fname)
            if fname.endswith('.json') and os.path.isfile(fpath):
                db_files.append(fname[:-5])
            else:
                log.warning("Ignoring invalid storage file %s", fpath)

        return db_files

    def _get_json_path(self, cluster_name):
        """
        Gets the path to the json storage file.
        """
        if not os.path.exists(self._storage_dir):
            os.makedirs(self._storage_dir)
        return os.path.join(self._storage_dir, cluster_name + ".json")

    def _clear_storage(self, db_path):
        """
        Clears a storage file.
        """
        if os.path.exists(db_path):
            os.unlink(db_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
# @(#)setup.py
#
#
# Copyright (C) 2013, GC3, University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__docformat__ = 'reStructuredText'

import os
import sys
import shutil

from setuptools.command import sdist
del sdist.finders[:]

ANSIBLE_PB_DIR = 'elasticluster/providers/ansible-playbooks'

def ansible_pb_files():
    basedir = os.path.dirname(__file__)
    ansible_data = [('share/elasticluster/etc', ['docs/config.template.ini'])]
    for (dirname, dirnames, filenames) in os.walk(ANSIBLE_PB_DIR):
        tmp = []
        for fname in filenames:
            if fname.startswith('.git'): continue
            tmp.append(os.path.join(dirname, fname))
        ansible_data.append((os.path.join('share', dirname), tmp))
    return ansible_data

from setuptools import setup, find_packages
setup(
    name = "elasticluster",
    version = "0.1",
    packages = find_packages(),
    install_requires = [
        'boto',
        'PyCLI',
        'paramiko',
        'ansible',  # works from pip, does not work from setup.py
        ],
    data_files = ansible_pb_files(),
    entry_points = {
        'console_scripts': [
            'elasticluster = elasticluster.main:main',
            ]
        },
    )

if __name__ == "__main__":
    if sys.argv[1] in ['develop', 'install']:
        develop = True if sys.argv[1] == 'develop' else False
        curdir = os.path.abspath(os.path.dirname(__file__))
        sharedir = os.path.join(os.path.abspath(sys.prefix), 'share', 'elasticluster')
        etcdir = os.path.join(sharedir, 'etc')
        templatecfg = os.path.join(curdir, 'docs', 'config.template.ini')
        templatedest = os.path.join(etcdir, os.path.basename(templatecfg))
        ansibledest = os.path.join(sharedir, 'providers', 'ansible-playbooks')
        ansiblesrc = os.path.join(curdir, 'elasticluster', 'providers', 'ansible-playbooks')

        if not os.path.exists(sharedir):
            os.makedirs(sharedir)
            
        if not os.path.exists(etcdir):
            os.makedirs(etcdir)

        if not os.path.exists(os.path.dirname(ansibledest)):
            os.makedirs(os.path.dirname(ansibledest))

        if not os.path.exists(ansibledest):
            if develop:
                os.symlink(ansiblesrc, ansibledest)
            else:
                shutil.copytree(ansiblesrc, ansibledest)

        if not os.path.exists(templatedest):
            if develop:
                os.symlink(templatecfg, templatedest)
            else:
                shutil.copy(templatecfg, etcdir)



#! /usr/bin/env python
#
#   Copyright (C) 2013 GC3, University of Zurich
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'



import unittest
import os

from elasticluster.conf import Configuration, Configurator
from elasticluster.cluster import Node, Cluster, ClusterStorage
from elasticluster.providers import AbstractCloudProvider, AbstractSetupProvider
from elasticluster.providers.cloud_providers import BotoCloudProvider

from test import config_cloud_ec2_url, config_cloud_provider,\
    config_cloud_ec2_access_key, config_cloud_ec2_secret_key,\
    config_cloud_ec2_region, config_setup_provider, config_setup_playbook_path,\
    config_cloud_name, config_cluster_name, config_cluster_cloud,\
    config_cluster_frontend, config_cluster_compute,\
    config_login_user_key_public, config_login_user_key_name,\
    config_login_image_user, config_frontend_security_group,\
    config_frontend_image, config_frontend_image_userdata,\
    config_frontend_flavor,\
    config_compute_security_group, config_compute_image,\
    config_compute_image_userdata, config_compute_flavor,\
    config_storage_path, config_setup_name,\
    config_cluster_setup_provider, config_cluster_login, config_login_name,\
    config_login_image_user_sudo, config_login_image_sudo,\
    config_login_user_key_private, config_setup_frontend_groups,\
    config_setup_compute_groups
    

class TestConfigurator(unittest.TestCase):


    def test_create_cloud_provider(self):
        configurator = Configurator()
        cloud_provider = configurator.create_cloud_provider(config_cloud_name)
        
        assert cloud_provider
        assert isinstance(cloud_provider, AbstractCloudProvider)
        assert cloud_provider._url == config_cloud_ec2_url
        assert cloud_provider._region_name == config_cloud_ec2_region
        assert cloud_provider._access_key == config_cloud_ec2_access_key
        assert cloud_provider._secret_key == config_cloud_ec2_secret_key
        
    def test_create_cluster(self):
        configurator = Configurator()
        cluster = configurator.create_cluster(config_cluster_name)
        
        assert cluster
        assert isinstance(cluster, Cluster)
        assert cluster.name == config_cluster_name
        assert cluster._cloud == config_cluster_cloud
        assert isinstance(cluster._cloud_provider, AbstractCloudProvider)
        assert cluster._frontend == int(config_cluster_frontend)
        assert cluster._compute == int(config_cluster_compute)
        assert isinstance(cluster._setup_provider, AbstractSetupProvider)


    def test_create_node(self):
        configurator = Configurator()
        cloud_provider = BotoCloudProvider(config_cloud_ec2_url, config_cloud_ec2_region, config_cloud_ec2_access_key, config_cloud_ec2_secret_key)
        node_frontend = configurator.create_node(config_cluster_name, Node.frontend_type, cloud_provider, "node001")
        assert node_frontend
        assert isinstance(node_frontend, Node)
        assert node_frontend.name == "node001"
        assert node_frontend.type == Node.frontend_type
        assert node_frontend._cloud_provider == cloud_provider
        assert node_frontend.user_key_public == os.path.expanduser(config_login_user_key_public)
        assert node_frontend.user_key_name == os.path.expanduser(config_login_user_key_name)
        assert node_frontend.image_user == config_login_image_user
        assert node_frontend.security_group == config_frontend_security_group
        assert node_frontend.image == config_frontend_image
        assert node_frontend.image_userdata == config_frontend_image_userdata
        assert node_frontend.flavor == config_frontend_flavor
        
        node_compute = configurator.create_node(config_cluster_name, Node.compute_type, cloud_provider, "node001")
        assert node_compute
        assert isinstance(node_compute, Node)
        assert node_compute.name == "node001"
        assert node_compute.type == Node.compute_type
        assert node_compute._cloud_provider == cloud_provider
        assert node_compute.user_key_public == os.path.expanduser(config_login_user_key_public)
        assert node_compute.user_key_name == config_login_user_key_name
        assert node_compute.image_user == config_login_image_user
        assert node_compute.security_group == config_compute_security_group
        assert node_compute.image == config_compute_image
        assert node_compute.image_userdata == config_compute_image_userdata
        assert node_compute.flavor == config_compute_flavor
        
    def test_create_cluster_storage(self):
        configurator = Configurator()
        storage = configurator.create_cluster_storage()
        assert storage
        assert isinstance(storage, ClusterStorage)
        assert storage._storage_dir == config_storage_path
        
    def test_create_setup_provider(self):
        configurator = Configurator()
        setup_provider = configurator.create_setup_provider(config_setup_name, config_cluster_name)
        
        assert setup_provider
        assert isinstance(setup_provider, AbstractSetupProvider)
        

class TestConfiguration(unittest.TestCase):
    
    def test_read_cloud_section(self):        
        cloud = Configuration.Instance().read_cloud_section(config_cloud_name)
        assert cloud["provider"] == config_cloud_provider
        assert cloud["ec2_url"] == config_cloud_ec2_url
        assert cloud["ec2_access_key"] == config_cloud_ec2_access_key
        assert cloud["ec2_secret_key"] == config_cloud_ec2_secret_key
        assert cloud["ec2_region"] == config_cloud_ec2_region
        
        
    def test_read_cluster_section(self):
        cluster = Configuration.Instance().read_cluster_section(config_cluster_name)
        assert cluster["cloud"] == config_cluster_cloud
        assert cluster["setup_provider"] == config_cluster_setup_provider
        assert cluster["frontend"] == config_cluster_frontend
        assert cluster["compute"] == config_cluster_compute
        assert cluster["login"] == config_cluster_login
        
        
    def test_read_node_section(self):
        node_frontend = Configuration.Instance().read_node_section(config_cluster_name, Node.frontend_type)
        assert node_frontend["security_group"] == config_frontend_security_group
        assert node_frontend["image_id"] == config_frontend_image
        assert node_frontend["flavor"] == config_frontend_flavor
        
        node_compute = Configuration.Instance().read_node_section(config_cluster_name, Node.compute_type)
        assert node_compute["security_group"] == config_compute_security_group
        assert node_compute["image_id"] == config_compute_image
        assert node_compute["flavor"] == config_compute_flavor
        
    
    def test_read_login_section(self):
        login = Configuration.Instance().read_login_section(config_login_name)
        assert login["image_user"] == config_login_image_user
        assert login["image_user_sudo"] == config_login_image_user_sudo
        assert login["image_sudo"] == config_login_image_sudo
        assert login["user_key_name"] == config_login_user_key_name
        assert login["user_key_private"] == os.path.expanduser(config_login_user_key_private)
        assert login["user_key_public"] == os.path.expanduser(config_login_user_key_public)
        
    def test_read_setup_section(self):
        setup = Configuration.Instance().read_setup_section(config_setup_name, config_cluster_name)
        assert setup["provider"] == config_setup_provider
        assert setup["playbook_path"] == os.path.expanduser(os.path.expandvars(config_setup_playbook_path))
        assert setup["frontend_groups"] == config_setup_frontend_groups
        assert setup["compute_groups"] == config_setup_compute_groups
        
        
        